==> fred-backend-zone_1.0.0.orig.tar.gz: fred-backend-zone-1.0.0/src/impl/lib/wire_format.cc <==
/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/wire_format.hh"
#include "src/impl/lib/exception.hh"

#include <algorithm>
#include <limits>
#include <stdexcept>
#include <utility>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

WireFormat::WireFormat()
{ }

WireFormat& WireFormat::append(const AbsoluteFqdn& absolute_fqdn)
{
    const auto fqdn = *absolute_fqdn.get_fqdn();
    data_.reserve(data_.size() + fqdn.length() + 1);
    constexpr auto label_delimiter = '.';
    auto label_begin = std::begin(fqdn);
    while (true)
    {
        auto label_end = std::find(label_begin, std::end(fqdn), label_delimiter);
        if (label_end == std::end(fqdn))
        {
            throw_runtime_error([]() { return "label must be terminated by a dot"; });
        }
        constexpr auto max_label_length = 63;
        const auto label_length = label_end - label_begin;
        if (max_label_length < label_length)
        {
            throw_runtime_error([]() { return "label must be at most 63 characters long"; });
        }
        data_.push_back(label_length);
        if (label_length == 0)
        {
            if (fqdn.length() == 1)
            {
                return *this;
            }
            throw_runtime_error([]() { return "label can not be empty"; });
        }
        data_.insert(data_.end(), label_begin, label_end);
        label_begin = label_end + 1;
        if (label_begin == std::end(fqdn))
        {
            constexpr auto last_label_length = std::uint8_t{0};
            data_.push_back(last_label_length);
            return *this;
        }
        label_end = std::find(label_begin, std::end(fqdn), label_delimiter);
    }
}

WireFormat& WireFormat::append(const DnsKey& dns_key)
{
    const std::size_t wire_format_size =
            sizeof(*dns_key.flags()) +
            sizeof(*dns_key.protocol()) +
            sizeof(*dns_key.algorithm()) +
            dns_key.public_key().size();
    data_.reserve(data_.size() + wire_format_size);
    auto data_back_inserter = back_inserter(data_);
    data_back_inserter = static_cast<std::uint8_t>((*dns_key.flags()) / 0x100);
    data_back_inserter = static_cast<std::uint8_t>((*dns_key.flags()) % 0x100);
    data_back_inserter = *dns_key.protocol();
    data_back_inserter = *dns_key.algorithm();
    data_.insert(data_.end(), std::begin(dns_key.public_key()), std::end(dns_key.public_key()));
    return *this;
}

const std::vector<std::uint8_t>& WireFormat::data() const &
{
    return data_;
}

std::vector<std::uint8_t> WireFormat::data() &&
{
    return std::move(data_);
}

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred

==> fred-backend-zone_1.0.1.orig.tar.gz: fred-backend-zone-1.0.1/src/impl/lib/wire_format.cc <==
/*
 * Copyright (C) 2020-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/wire_format.hh"
#include "src/impl/lib/exception.hh"

#include <algorithm>
#include <limits>
#include <stdexcept>
#include <utility>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

WireFormat::WireFormat()
{ }

WireFormat& WireFormat::append(const AbsoluteFqdn& absolute_fqdn)
{
    const auto fqdn = *absolute_fqdn.get_fqdn();
    data_.reserve(data_.size() + fqdn.length() + 1);
    constexpr auto label_delimiter = '.';
    auto label_begin = std::begin(fqdn);
    while (true)
    {
        const auto label_end = std::find(label_begin, std::end(fqdn), label_delimiter);
        if (label_end == std::end(fqdn))
        {
            throw_runtime_error([]() { return "label must be terminated by a dot"; });
        }
        constexpr auto max_label_length = 63;
        const auto label_length = label_end - label_begin;
        if (max_label_length < label_length)
        {
            throw_runtime_error([]() { return "label must be at most 63 characters long"; });
        }
        data_.push_back(label_length);
        if (label_length == 0)
        {
            if (fqdn.length() == 1)
            {
                return *this;
            }
            throw_runtime_error([]() { return "label cannot be empty"; });
        }
        data_.insert(data_.end(), label_begin, label_end);
        label_begin = label_end + 1;
        if (label_begin == std::end(fqdn))
        {
            constexpr auto last_label_length = std::uint8_t{0};
            data_.push_back(last_label_length);
            return *this;
        }
    }
}

WireFormat& WireFormat::append(const DnsKey& dns_key)
{
    const std::size_t wire_format_size =
            sizeof(*dns_key.flags()) +
            sizeof(*dns_key.protocol()) +
            sizeof(*dns_key.algorithm()) +
            dns_key.public_key().size();
    data_.reserve(data_.size() + wire_format_size);
    auto data_back_inserter = back_inserter(data_);
    data_back_inserter = static_cast<std::uint8_t>((*dns_key.flags()) / 0x100);
    data_back_inserter = static_cast<std::uint8_t>((*dns_key.flags()) % 0x100);
    data_back_inserter = *dns_key.protocol();
    data_back_inserter = *dns_key.algorithm();
    data_.insert(data_.end(), std::begin(dns_key.public_key()), std::end(dns_key.public_key()));
    return *this;
}

const std::vector<std::uint8_t>& WireFormat::data() const &
{
    return data_;
}

std::vector<std::uint8_t> WireFormat::data() &&
{
    return std::move(data_);
}

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred
